'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.TemplateBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.applyTemplate = applyTemplate;
exports.template = template;

var _types = require('../helper/types.js');

var _property = require('./property.js');

var _Builder2 = require('./Builder');

var _Builder3 = _interopRequireDefault(_Builder2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * The counter is used to generate unique DOM's id.
 * @type {number}
 */
var counter = 0;

/**
 * The attribute name hosting the old light node id.
 * @type {string}
 */
var OLD_CONTENT_ID_ATTR_NAME = 'ceb-old-content-id';

/**
 * Regex to detect the *ceb-content* attributes.
 * @type {RegExp}
 */
var CONTENT_ATTR_REG_EX = /ceb\-content/im;

/**
 * Regex to detect the *content* element.
 * @type {RegExp}
 */
var CONTENT_NODE_REG_EX = /<content><\/content>/im;

/**
 * @param {!string} html the HTML template
 * @returns {boolean} true if the HTML template handle a light DOM node
 */
function hasContent(html) {
    return html.search(CONTENT_ATTR_REG_EX) !== -1 || html.search(CONTENT_NODE_REG_EX) !== -1;
}

/**
 * Update or replace an eventual content flag according to the given id.
 * @param {!string} html the HTML template
 * @param {!string} newCebContentId the new content node id
 * @returns {string} the updated HTML template
 */
function replaceContent(html, newCebContentId) {
    return html.replace('<content></content>', '<ceb-lightdom ceb-content></ceb-lightdom>').replace('ceb-content', newCebContentId);
}

/**
 * Try to find a light DOM node
 * @param {!HTMLElement} el the custom element
 * @returns {HTMLElement} the light DOM node if found otherwise it's the given HTML Element
 */
function findContentNode(el) {
    if (!el) {
        return;
    }
    var oldCebContentId = el.getAttribute(OLD_CONTENT_ID_ATTR_NAME);
    if (oldCebContentId) {
        return findContentNode(el.querySelector('[' + oldCebContentId + ']')) || el;
    }
    return el;
}

/**
 * Remove and return the children of the light DOM node.
 * @param {!HTMLElement} el the custom element
 * @returns {DocumentFragment} the light DOM fragment
 */
function cleanOldContentNode(el) {
    var oldContentNode = el.lightDOM,
        lightFrag = document.createDocumentFragment();
    while (oldContentNode.childNodes.length > 0) {
        lightFrag.appendChild(oldContentNode.removeChild(oldContentNode.childNodes[0]));
    }
    return lightFrag;
}

/**
 * Apply the template to the element.
 * @param {!HTMLElement} el the custom element
 * @param {!string} tpl the template
 */
function applyTemplate(el, tpl) {
    var lightFrag = void 0,
        handleContentNode = hasContent(tpl);

    if (handleContentNode) {
        var newCebContentId = 'ceb-content-' + counter++;
        lightFrag = cleanOldContentNode(el);

        tpl = replaceContent(tpl, newCebContentId);

        el.setAttribute(OLD_CONTENT_ID_ATTR_NAME, newCebContentId);
    }

    el.innerHTML = tpl;

    if (handleContentNode && lightFrag) {
        el.lightDOM.appendChild(lightFrag);
    }
}

/**
 * The template builder.
 * Its goal is to provide a way to fill the content of a custom element.
 */

var TemplateBuilder = exports.TemplateBuilder = function (_Builder) {
    _inherits(TemplateBuilder, _Builder);

    /**
     * @param {!string|function(el: HTMLElement)} tpl the template as a string or a function
     */
    function TemplateBuilder(tpl) {
        _classCallCheck(this, TemplateBuilder);

        /**
         * @ignore
         */
        var _this = _possibleConstructorReturn(this, (TemplateBuilder.__proto__ || Object.getPrototypeOf(TemplateBuilder)).call(this));

        _this.data = { tpl: tpl };
        return _this;
    }

    /**
     * Logic of the builder.
     * @param {!ElementBuilder.context.p} proto the prototype
     * @param {!ElementBuilder.on} on the method on
     */


    _createClass(TemplateBuilder, [{
        key: 'build',
        value: function build(proto, on) {
            var data = this.data;

            (0, _property.property)('lightDOM').getter(function (el) {
                return findContentNode(el);
            }).build(proto, on);

            on('before:createdCallback').invoke(function (el) {
                applyTemplate(el, (0, _types.isFunction)(data.tpl) ? data.tpl(el) : data.tpl);
            });
        }
    }]);

    return TemplateBuilder;
}(_Builder3.default);

/**
 * Get a new template builder.
 * @param {!string|function} tpl the string or function template
 * @returns {TemplateBuilder} the template builder
 */


function template(tpl) {
    return new TemplateBuilder(tpl);
}